@@ -38,7 +38,7 @@ namespace QPulseAudio
 
 class Server;
 
-class Q_DECL_EXPORT Context : public QObject
+class Context : public QObject
 {
     Q_OBJECT
 public:
@@ -62,8 +62,11 @@ public:
     const SourceOutputMap &sourceOutputs() const { return m_sourceOutputs; }
     const ClientMap &clients() const { return m_clients; }
     const CardMap &cards() const { return m_cards; }
+    const ModuleMap &modules() const { return m_modules; }
     const StreamRestoreMap &streamRestores() const { return m_streamRestores; }
     Server *server() const { return m_server; }
+    QString newDefaultSink() const { return m_newDefaultSink; }
+    QString newDefaultSource() const { return m_newDefaultSource; }
 
     void subscribeCallback(pa_context *context, pa_subscription_event_type_t type, uint32_t index);
     void contextStateCallback(pa_context *context);
@@ -74,6 +77,7 @@ public:
     void sourceOutputCallback(const pa_source_output_info *info);
     void clientCallback(const pa_client_info *info);
     void cardCallback(const pa_card_info *info);
+    void moduleCallback(const pa_module_info *info);
     void streamRestoreCallback(const pa_ext_stream_restore_info *info);
     void serverCallback(const pa_server_info *info);
 
@@ -92,8 +96,9 @@ public:
         newVolume = qBound<qint64>(0, newVolume, PA_VOLUME_MAX);
         pa_cvolume newCVolume = cVolume;
         if (channel == -1) { // -1 all channels
+            const qint64 diff = newVolume - pa_cvolume_avg(&cVolume);
             for (int i = 0; i < newCVolume.channels; ++i) {
-                newCVolume.values[i] = newVolume;
+                newCVolume.values[i] = qBound<qint64>(0, newCVolume.values[i] + diff, PA_VOLUME_MAX);
             }
         } else {
             Q_ASSERT(newCVolume.channels > channel);
@@ -162,12 +167,16 @@ private:
     SourceOutputMap m_sourceOutputs;
     ClientMap m_clients;
     CardMap m_cards;
+    ModuleMap m_modules;
     StreamRestoreMap m_streamRestores;
     Server *m_server;
 
     pa_context *m_context;
     pa_glib_mainloop *m_mainloop;
 
+    QString m_newDefaultSink;
+    QString m_newDefaultSource;
+
     int m_references;
     static Context* s_context;
 };
