@@ -82,9 +82,13 @@ QString StreamRestore::device() const
 void StreamRestore::setDevice(const QString &device)
 {
     if (m_cache.valid) {
-        writeChanges(m_cache.volume, m_cache.channelMap, m_cache.muted, device);
+        if (m_cache.device != device) {
+            writeChanges(m_cache.volume, m_cache.channelMap, m_cache.muted, device);
+        }
     } else {
-        writeChanges(m_volume, m_channelMap, m_muted, device);
+        if (m_device != device) {
+            writeChanges(m_volume, m_channelMap, m_muted, device);
+        }
     }
 }
 
@@ -115,9 +119,13 @@ bool StreamRestore::isMuted() const
 void StreamRestore::setMuted(bool muted)
 {
     if (m_cache.valid) {
-        writeChanges(m_cache.volume, m_cache.channelMap, muted, m_cache.device);
+        if (m_cache.muted != muted) {
+            writeChanges(m_cache.volume, m_cache.channelMap, muted, m_cache.device);
+        }
     } else {
-        writeChanges(m_volume, m_channelMap, muted, m_device);
+        if (m_muted != muted) {
+            writeChanges(m_volume, m_channelMap, muted, m_device);
+        }
     }
 }
 
@@ -136,9 +144,9 @@ QStringList StreamRestore::channels() const
     return m_channels;
 }
 
-QList<qint64> StreamRestore::channelVolumes() const
+QList<qreal> StreamRestore::channelVolumes() const
 {
-    QList<qint64> ret;
+    QList<qreal> ret;
     ret.reserve(m_volume.channels);
     for (int i = 0; i < m_volume.channels; ++i) {
         ret << m_volume.values[i];
